# Emissary

[Emissary Ingress](https://github.com/datawire/ambassador) - Emissary Ingress is a self-service, comprehensive API gateway that is Kubernetes-native and built on [Envoy Proxy](https://www.envoyproxy.io/).

## TL;DR;

```console
$ helm repo add emissary-ingress https://app.getambassador.io
$ helm repo update
$ helm install emissary-ingress --devel emissary-ingress/emissary-ingress -n ambassador
```

## Introduction

This chart deploys Emissary Ingress on a [Kubernetes](http://kubernetes.io) cluster using the [Helm](https://helm.sh) package manager.

This chart is used to install the 2.0 release line of Emissary Ingress. 

Versions in the older 1.0 release line of Emissary Ingress and Ambassador Edge Stack share a
single chart that can be found in this repository under the branch for the specific release, e.g.
[the release/v1.14 branch] for the latest 1.14 chart, [release/v1.13] for 1.13, and so on.

> Note that for 1.0 releases, the `enableAES` helm value is used to control installing Edge Stack or
  Emissary Ingress.

As of version 2.0, Emissary Ingress and Ambassador Edge Stack have separate charts. The helm chart
for Ambassador Edge Stack 2.0 lives in the [Ambassador Edge Stack chart repository].

See the [Ambassador Edge Stack FAQ] for more information about the differences between Emissary
Ingress and Ambassador Edge Stack.

[the release/v1.14 branch]: https://github.com/emissary-ingress/emissary/tree/release/v1.14/charts/ambassador 
[release/v1.13]: https://github.com/emissary-ingress/emissary/tree/release/v1.13/charts/ambassador 
[Ambassador Edge Stack chart repository]: https://github.com/datawire/edge-stack/tree/main/charts/edge-stack
[Ambassador Edge Stack FAQ]: https://www.getambassador.io/docs/edge-stack/latest/about/faq/#whats-the-difference-between-ossproductname-and-aesproductname

## Prerequisites

- Kubernetes v1.11+

## Installing the Chart

To install the chart with the release name `emissary-ingress`:

```console
$ helm install emissary-ingress --devel emissary-ingress/emissary-ingress -n ambassador
```

The command deploys Emissary Ingress on the Kubernetes cluster in the default configuration. The [configuration](#configuration) section lists the parameters that can be configured during installation.

> **Tip**: List all releases using `helm list`

## Uninstalling the Chart

To uninstall/delete the `emissary-ingress`:

```console
$ helm delete emissary-ingress -n ambassador
```

The command removes all the Kubernetes components associated with the chart and deletes the release.

## Changelog

Notable chart changes are listed in the [CHANGELOG](./CHANGELOG.md)

## Configuration

The following table lists the configurable parameters of the `emissary-ingress` chart and their default values.

|               Parameter               |                                                                                                                                                                                                                                                                                                                                             Description                                                                                                                                                                                                                                                                                                                                             |                                                                                                                                    Default                                                                                                                                     |
|---------------------------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| nameOverride                          | Override the generated chart name. Defaults to .Chart.Name.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         | `''`                                                                                                                                                                                                                                                                           |
| fullnameOverride                      | Override the generated release name. Defaults to .Release.Name.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `''`                                                                                                                                                                                                                                                                           |
| namespaceOverride                     | Override the generated release namespace. Defaults to .Release.Namespace.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           | `''`                                                                                                                                                                                                                                                                           |
| replicaCount                          | Number of Ambassador replicas                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | `3`                                                                                                                                                                                                                                                                            |
| daemonSet                             | If true, Create a DaemonSet. By default Deployment controller will be created                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | `false`                                                                                                                                                                                                                                                                        |
| barePod                               | If true, Create a bare Pod instead of a Deployment or DaemonSet; for test purposes.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 | `false`                                                                                                                                                                                                                                                                        |
| test.enabled                          | If true, Create test Pod to verify the Ambassador service works correctly (Only created on helm test)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | `false`                                                                                                                                                                                                                                                                        |
| test.image                            | Image to use for the test Pod                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | `busybox`                                                                                                                                                                                                                                                                      |
| canary.enabled                        | Creates a Canary deployment and service using the following canary settings. Other settings inherited from main deployment                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          | `false`                                                                                                                                                                                                                                                                        |
| canary.mixPods                        | When true allows for requests to the main ambassador to hit either the pods from the main deployment or the canary deployment. Requests to the canary service will still only hit the canary pods                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   | `false`                                                                                                                                                                                                                                                                        |
| canary.replicaCount                   | Similar to 'replicaCount', but controls the number of pods for the Canary deployment. 0 by default                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  | `0`                                                                                                                                                                                                                                                                            |
| canary.image.repository               | Controlls the image repository for the canary deployment.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           | `''`                                                                                                                                                                                                                                                                           |
| canary.image.tag                      | Controlls the tag for the canary deployments image.repository. Uses the repository from the main deployment if the canay.image.repository is not set.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | `''`                                                                                                                                                                                                                                                                           |
| canary.envRaw                         | Environment variables to be used for the canary deployment, see `envRaw` for usage example                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          | `{}`                                                                                                                                                                                                                                                                           |
| ingressClassResource.enabled          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `true`                                                                                                                                                                                                                                                                         |
| ingressClassResource.name             |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `ambassador`                                                                                                                                                                                                                                                                   |
| ingressClassResource.default          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `false`                                                                                                                                                                                                                                                                        |
| ingressClassResource.controllerValue  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `getambassador.io/ingress-controller`                                                                                                                                                                                                                                          |
| autoscaling                           | Enable autoscaling using HorizontalPodAutoscaler daemonSet: true, autoscaling will be disabled                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      | `{"enabled":false,"maxReplicas":5,"metrics":[{"resource":{"name":"cpu","target":{"averageUtilization":60,"type":"Utilization"}},"type":"Resource"},{"resource":{"name":"memory","target":{"averageUtilization":60,"type":"Utilization"}},"type":"Resource"}],"minReplicas":2}` |
| podDisruptionBudget                   |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `{}`                                                                                                                                                                                                                                                                           |
| namespace.name                        | Explicitly set the AMBASSADOR_NAMESPACE environment variable                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        | ``                                                                                                                                                                                                                                                                             |
| env                                   | Any additional environment variables for ambassador pods <br> Example: <br> `Exposing statistics via StatsD` <br> `STATSD_ENABLED: true` <br> `STATSD_HOST: statsd-sink` <br> `sets the minimum number of seconds between Envoy restarts` <br> `AMBASSADOR_RESTART_TIME: 15` <br> `sets the number of seconds that the Envoy will wait for open connections to drain on a restart` <br> `AMBASSADOR_DRAIN_TIME: 5` <br> `sets the number of seconds that Ambassador will wait for the old Envoy to clean up and exit on a restart` <br> `AMBASSADOR_SHUTDOWN_TIME: 10` <br> `labels Ambassador with an ID to allow for configuring multiple Ambassadors in a cluster` <br> `AMBASSADOR_ID: default` | `{}`                                                                                                                                                                                                                                                                           |
| envRaw                                | Additional container environment variable in raw YAML format <br> Example: <br> `envRaw: |` <br> `- name: REDIS_PASSWORD` <br> `value: password` <br> `valueFrom:` <br> `secretKeyRef:` <br> `name: redis-password` <br> `key: password` <br> `- name: POD_IP` <br> `valueFrom:` <br> `fieldRef:` <br> `fieldPath: status.podIP`                                                                                                                                                                                                                                                                                                                                                                    | `{}`                                                                                                                                                                                                                                                                           |
| imagePullSecrets                      | Image pull secrets                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  | `[]`                                                                                                                                                                                                                                                                           |
| security.podSecurityContext           | Security Context for all containers in the pod. https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#podsecuritycontext-v1-core                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `{"runAsUser":8888}`                                                                                                                                                                                                                                                           |
| security.containerSecurityContext     | Security Context for the Ambassador container specifically https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#securitycontext-v1-core                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             | `{"allowPrivilegeEscalation":false}`                                                                                                                                                                                                                                           |
| security.podSecurityPolicy            | A basic PodSecurityPolicy to ensure Ambassador is running with appropriate security permissions https://kubernetes.io/docs/concepts/policy/pod-security-policy/ A set of reasonable defaults is outlined below. This is not created by default as it should only be created by a one Release. If you want to use the PodSecurityPolicy in the chart, create it in the "master" Release and then leave it unset in all others. Set the `rbac.podSecurityPolicies` in all non-"master" Releases.                                                                                                                                                                                                      | `{}`                                                                                                                                                                                                                                                                           |
| image.repository                      | Emissary Ingress docker repo                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        | `docker.io/emissaryingress/emissary`                                                                                                                                                                                                                                           |
| image.tag                             | Emissary Ingress docker tag                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         | `3.0.0`                                                                                                                                                                                                                                                                        |
| image.pullPolicy                      | Pod container image pull policy                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `IfNotPresent`                                                                                                                                                                                                                                                                 |
| dnsPolicy                             | Dns policy, when hostNetwork set to ClusterFirstWithHostNet                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         | `ClusterFirst`                                                                                                                                                                                                                                                                 |
| hostNetwork                           | If true, uses the host network, useful for on-premise setups                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        | `false`                                                                                                                                                                                                                                                                        |
| service.type                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `LoadBalancer`                                                                                                                                                                                                                                                                 |
| service.portsRaw                      | For test purposes, do not use.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      | `''`                                                                                                                                                                                                                                                                           |
| service.externalTrafficPolicy         |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | ``                                                                                                                                                                                                                                                                             |
| service.sessionAffinity               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | ``                                                                                                                                                                                                                                                                             |
| service.sessionAffinityConfig         |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | ``                                                                                                                                                                                                                                                                             |
| service.externalIPs                   |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `[]`                                                                                                                                                                                                                                                                           |
| service.annotations                   |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `{}`                                                                                                                                                                                                                                                                           |
| service.nameOverride                  | Manually set the name of the generated Service                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      | ``                                                                                                                                                                                                                                                                             |
| adminService.create                   | If true, create a service for Ambassador's admin UI                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 | `true`                                                                                                                                                                                                                                                                         |
| adminService.type                     | Ambassador's admin service type to be used                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          | `ClusterIP`                                                                                                                                                                                                                                                                    |
| adminService.port                     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `8877`                                                                                                                                                                                                                                                                         |
| adminService.snapshotPort             |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `8005`                                                                                                                                                                                                                                                                         |
| adminService.nodePort                 | If explicit NodePort for admin service is required                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  | ``                                                                                                                                                                                                                                                                             |
| adminService.annotations              | Annotations to apply to Ambassador admin service                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    | `{}`                                                                                                                                                                                                                                                                           |
| adminService.loadBalancerIP           | IP address to assign (if cloud provider supports it)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                | ``                                                                                                                                                                                                                                                                             |
| adminService.loadBalancerSourceRanges | Passed to cloud provider load balancer if created (e.g: AWS ELB)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    | ``                                                                                                                                                                                                                                                                             |
| rbac.create                           | Specifies whether RBAC resources should be created                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  | `true`                                                                                                                                                                                                                                                                         |
| rbac.podSecurityPolicies              | List of Pod Security Policies to use on the container.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              | `[]`                                                                                                                                                                                                                                                                           |
| rbac.nameOverride                     | Name of the RBAC resources defaults to the name of the release. Set nameOverride when installing Ambassador with cluster-wide scope in different namespaces with the same release name to avoid conflicts.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          | ``                                                                                                                                                                                                                                                                             |
| scope.singleNamespace                 | Set the AMBASSADOR_SINGLE_NAMESPACE environment variable and create namespaced RBAC if rbac.enabled: true                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           | `false`                                                                                                                                                                                                                                                                        |
| serviceAccount.create                 | Specifies whether a service account should be created                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | `true`                                                                                                                                                                                                                                                                         |
| serviceAccount.name                   | The name of the service account to use. If not set and create is true, a name is generated using the fullname template                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              | ``                                                                                                                                                                                                                                                                             |
| serviceAccount.extra                  | Extra YAML to include at the end of the ServiceAccount                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              | `''`                                                                                                                                                                                                                                                                           |
| deploymentStrategy.type               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `RollingUpdate`                                                                                                                                                                                                                                                                |
| progressDeadlines.ambassador          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `600`                                                                                                                                                                                                                                                                          |
| progressDeadlines.agent               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `600`                                                                                                                                                                                                                                                                          |
| restartPolicy                         |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | ``                                                                                                                                                                                                                                                                             |
| terminationGracePeriodSeconds         |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | ``                                                                                                                                                                                                                                                                             |
| initContainers                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `[]`                                                                                                                                                                                                                                                                           |
| sidecarContainers                     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `[]`                                                                                                                                                                                                                                                                           |
| livenessProbe                         | Liveness probe for emissary pods                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    | `{"failureThreshold":3,"initialDelaySeconds":30,"periodSeconds":3}`                                                                                                                                                                                                            |
| readinessProbe                        | Liveness probe for emissary pods                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    | `{"failureThreshold":3,"initialDelaySeconds":30,"periodSeconds":3}`                                                                                                                                                                                                            |
| volumes                               |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `[]`                                                                                                                                                                                                                                                                           |
| volumeMounts                          |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `[]`                                                                                                                                                                                                                                                                           |
| podLabels                             | Additional labels for ambassador pods                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | `{}`                                                                                                                                                                                                                                                                           |
| podAnnotations                        | Additional annotations for ambassador pods <br> Example: <br> `prometheus.io/scrape: "true"` <br> `prometheus.io/port: "9102"`                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      | `{}`                                                                                                                                                                                                                                                                           |
| deploymentLabels                      | Additional labels for ambassador DaemonSet/Deployment                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | `{}`                                                                                                                                                                                                                                                                           |
| deploymentAnnotations                 | Additional annotations for ambassador DaemonSet/Deployment <br> Example: <br> `configmap.reloader.stakater.com/auto: "true"`                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        | `{}`                                                                                                                                                                                                                                                                           |
| resources                             | CPU/memory resource requests/limits                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 | `{"limits":{"cpu":"1000m","memory":"600Mi"},"requests":{"cpu":"200m","memory":"300Mi"}}`                                                                                                                                                                                       |
| priorityClassName                     | The name of the priorityClass for the ambassador DaemonSet/Deployment                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | `''`                                                                                                                                                                                                                                                                           |
| nodeSelector                          | NodeSelector for ambassador pods                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    | `{}`                                                                                                                                                                                                                                                                           |
| tolerations                           |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `[]`                                                                                                                                                                                                                                                                           |
| affinity                              | Affinity for ambassador pods                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        | `{}`                                                                                                                                                                                                                                                                           |
| topologySpreadConstraints             | Topology Spread Constraints for Ambassador pods. Stable since 1.19.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 | `[]`                                                                                                                                                                                                                                                                           |
| ambassadorConfig                      | Config thats mounted to `/ambassador/ambassador-config`                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             | `''`                                                                                                                                                                                                                                                                           |
| metrics.serviceMonitor.enabled        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `false`                                                                                                                                                                                                                                                                        |
| resolvers                             | Resolvers are used to configure the discovery service strategy for Ambasador See: https://www.getambassador.io/docs/edge-stack/latest/topics/running/resolvers/ <br> Example: <br> `Configuration for a Consul Resolver` <br> `address: consul-server.default.svc.cluster.local:8500` <br> `datacenter: dc1`                                                                                                                                                                                                                                                                                                                                                                                        | `{"consul":{"create":false,"name":"consul-dc1","spec":{}},"endpoint":{"create":false,"name":"endpoint"}}`                                                                                                                                                                      |
| module.diagnostics.enabled            |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `false`                                                                                                                                                                                                                                                                        |
| prometheusExporter                    | DEPRECATED: Enabling the prometheus exporter creates a sidecar and configures ambassador to use it Ambassador now exposes the /metrics endpoint in Envoy. See https://www.getambassador.io/user-guide/monitoring#deployment for more information on how to use the /metrics endpoint                                                                                                                                                                                                                                                                                                                                                                                                                | `{"enabled":false,"pullPolicy":"IfNotPresent","repository":"prom/statsd-exporter","resources":{},"tag":"v0.8.1"}`                                                                                                                                                              |
| agent.enabled                         | If `true`, installs the ambassador-agent Deployment, ServiceAccount and ClusterRole in the ambassador namespace, enabling the Service Catalog                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | `true`                                                                                                                                                                                                                                                                         |
| agent.cloudConnectToken               | API token for reporting snapshots to the [Service Catalog](https://app.getambassador.io/cloud/catalog/); If empty, agent will not report snapshots                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  | `''`                                                                                                                                                                                                                                                                           |
| agent.rpcAddress                      | Address of the ambassador Service Catalog rpc server.                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               | `https://app.getambassador.io/`                                                                                                                                                                                                                                                |
| agent.createArgoRBAC                  |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `true`                                                                                                                                                                                                                                                                         |
| agent.image.tag                       | Leave blank to use image.repository and image.tag                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   | ``                                                                                                                                                                                                                                                                             |
| agent.image.repository                |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | ``                                                                                                                                                                                                                                                                             |
| deploymentTool                        |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `''`                                                                                                                                                                                                                                                                           |
| docker                                | configure docker to pull from private registry                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      | `{}`                                                                                                                                                                                                                                                                           |
| createNamespace                       |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     | `false`                                                                                                                                                                                                                                                                        |
| createDefaultListeners                | Whether Emissary should be created with default listeners: - HTTP on port 8080 - HTTPS on port 8443                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 | `false`                                                                                                                                                                                                                                                                        |
| lifecycle                             | Configure a 'lifecycle' section for the main pod                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    | ``                                                                                                                                                                                                                                                                             |


Specify each parameter using the `--set key=value[,key=value]` argument to `helm install`. For example:

```console
$ helm install emissary-ingress --devel emissary-ingress/emissary-ingress -n ambassador --set nameOverride=''
```

Alternatively, a YAML file that specifies the values for the parameters can be provided while
installing the chart. For example:

```console
$ helm install emissary-ingress --devel emissary-ingress/emissary-ingress -n ambassador --values values.yaml
```
